"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Description = void 0;
const core_1 = require("@tsed/core");
const jsonEntityFn_1 = require("./jsonEntityFn");
/**
 * Add a description to the class, method or property
 *
 * ## Examples
 * ### On class
 *
 * ```typescript
 * @Description("description")
 * class Model {
 *
 * }
 * ```
 *
 * ### On method
 *
 * ```typescript
 * @Controller("/")
 * class ModelCtrl {
 *    @Description("description")
 *    async method() {}
 * }
 * ```
 *
 * ### On parameter
 *
 * ```typescript
 * @Controller("/")
 * class ModelCtrl {
 *    async method(@Description("description") @PathParam("id") id: string) {}
 * }
 * ```
 *
 * ### On property
 *
 * ```typescript
 * class Model {
 *    @Description("description")
 *    id: string;
 * }
 * ```
 *
 * @param {string} description
 * @decorator
 * @validation
 * @swagger
 * @schema
 * @input
 * @classDecorator
 * @methodDecorator
 * @propertyDecorator
 * @parameterDecorator
 */
function Description(description) {
    return (0, jsonEntityFn_1.JsonEntityFn)((store, args) => {
        switch ((0, core_1.decoratorTypeOf)(args)) {
            case core_1.DecoratorTypes.PROP:
            case core_1.DecoratorTypes.CLASS:
                store.schema.description(description);
                break;
            case core_1.DecoratorTypes.PARAM:
                store.parameter.description(description);
                break;
            case core_1.DecoratorTypes.METHOD:
                store.operation?.description(description);
                break;
        }
    });
}
exports.Description = Description;
//# sourceMappingURL=description.js.map