import { Type } from "@tsed/core";
import { OS3Example } from "@tsed/openspec";
import { JsonSchema, JsonSchemaObject } from "../../domain/JsonSchema";
import { JsonHeader, JsonHeaders } from "../../interfaces/JsonOpenSpec";
import { GenericValue } from "../../utils/generics";
export interface ReturnsChainedDecorators {
    <T>(target: Object, propertyKey: string | symbol, descriptor: TypedPropertyDescriptor<T>): TypedPropertyDescriptor<T> | void;
    (target: Function): void;
    /**
     * Set a Content-Type for the current response
     * @param value
     * @constructor
     */
    ContentType(value: string): this;
    /**
     * Add a description
     * @param description
     */
    Description(description: string): this;
    /**
     * Add examples
     * @param examples
     */
    Examples(examples: Record<string, OS3Example>): this;
    Examples(examples: Record<string, any>): this;
    Examples(examples: any): this;
    /**
     * Change the model type
     * @param type
     */
    Type(type: Type<any> | any): this;
    /**
     * Change the status
     * @param status
     * @constructor
     */
    Status(status: string | number): this;
    /**
     * Add the nested types
     * @param types
     */
    Of(...types: GenericValue[]): this;
    /**
     * For the integer type
     */
    OfInteger(): this;
    /**
     * Add the nested types
     * @param types
     */
    OneOf(...types: GenericValue[]): this;
    /**
     * Add the nested types
     * @param types
     */
    AllOf(...types: GenericValue[]): this;
    /**
     * Add the nested types
     * @param types
     */
    AnyOf(...types: GenericValue[]): this;
    /**
     * Declare a nested generic models
     * @param generics
     */
    Nested(...generics: GenericValue[]): this;
    /**
     * Add header.
     * @param key
     * @param value
     */
    Header(key: string, value: number | string | (JsonHeader & {
        value?: string | number | boolean;
    })): this;
    /**
     * Add headers
     */
    Headers(headers: JsonHeaders): this;
    /**
     * Add location hea
     */
    Location(location: string, meta?: JsonHeader): this;
    /**
     * Assign partial schema
     * @param schema
     */
    Schema(schema: Partial<JsonSchemaObject> | JsonSchema): this;
    /**
     * Add an inline title for the return model.
     * @param title
     */
    Title(title: string): this;
    /**
     * Use group to filter model
     * @param groups
     */
    Groups(...groups: string[]): this;
    Groups(groupName: string, groups: string[]): this;
    /**
     * Add a list of allowed groups to filter dynamically fields. Listed groups can be used by the consumer to change
     * the mapped response.
     * @param allowedGroups
     */
    AllowedGroups(...allowedGroups: string[]): this;
    [key: string]: any;
}
/**
 * Add responses documentation for a specific status code.
 *
 * ## Usage
 *
 * Ts.ED v5/v6 API introducing the chaining decorator concept. Now a decorator like Returns can be used with another decorators like Description.
 *
 * ::: warning
 * v5 has a basic support of the chaining decorator to facilitate the migration to v6!
 * :::
 *
 * ```typescript
 * import {Returns} from "@tsed/schema";
 *
 * @Controller("/")
 * class MyController {
 *   @Returns(404, String).Description("Not Found")
 *   @Returns(200, Model).Description("Success")
 *   async myMethod(): Promise<Model> {}
 * }
 * ```
 *
 * ::: tip
 * TypeScript and your IDE will discover automatically the chained decorators. But for more details you can look on @@ReturnsChainedDecorators@@ interface, to know
 * what chained decorators are available under Returns decorator.
 * :::
 *
 * This example will produce this documentation in swagger:
 *
 * ```json
 * {
 *   "responses": {
 *     "404": {
 *       "description": "Description",
 *       "schema": {"type": "string"}
 *     },
 *     "2OO": {
 *       "description": "Description",
 *       "schema": {"$ref": "..."}
 *     }
 *   }
 * }
 * ```
 *
 * ## Declaring an Array
 *
 * Use chained decorators to declare an array with model as following:
 *
 * ```typescript
 * import {Returns} from "@tsed/schema";
 *
 * @Controller("/models")
 * class ModelCtrl {
 *   @Get("/")
 *   @Returns(200, Array).Of(Model).Description("Success")
 *   async myMethod(): Promise<Model>  {
 *   }
 * }
 * ```
 *
 * Deprecated version:
 *
 * ```typescript
 * import {ReturnsArray} from "@tsed/common";
 * import {Returns} from "@tsed/schema";
 *
 * @Controller("/")
 * class MyController {
 *   @ReturnsArray(200, Model) // deprecated
 *   async myMethod(): Promise<Model> {}
 * }
 * ```
 *
 * ### Declaring a generic model <Badge text="6+"/>
 *
 * Sometime, it might be useful to use generic models. TypeScript doesn't store the generic type in the metadata. This is why we need to
 * declare explicitly the generic models with the decorators.
 *
 * One of the generic's usage, can be a paginated list. With Returns decorator it's now possible to declare generic type and generate the appropriate OpenSpec documentation.
 *
 * Starting with the pagination model, by using @@Generics@@ and @@CollectionOf@@:
 *
 * ```typescript
 * @Generics("T")
 * class Pagination<T> {
 *  @CollectionOf("T")
 *  data: T[];
 *
 *  @Property()
 *  totalCount: number;
 * }
 * ```
 *
 * Now, we need a model to be used with the generic Pagination model:
 *
 * ```typescript
 * class Product {
 *  @Property()
 *  id: string;
 *
 *  @Property()
 *  title: string;
 * }
 * ```
 *
 * Finally, we can use our models on a method as following:
 *
 * ```typescript
 * class Controller {
 *   @OperationPath("POST", "/")
 *   @Returns(200, Pagination).Of(Product).Description("description")
 *   async method(): Promise<Pagination<Product> | null> {
 *     return null;
 *   }
 * }
 * ```
 *
 * ### Declaring a nested generics models <Badge text="6+"/>
 *
 * It's also possible to declare a nested generics models in order to have this type `Pagination<Submission<Product>>`:
 *
 * ```typescript
 * import {Generics, Property, Returns} from "@tsed/schema";
 * import {Post} from "@tsed/common";
 *
 * class Controller {
 *   @Post("/")
 *   @Returns(200, Pagination).Of(Submission).Nested(Product).Description("description")
 *   async method(): Promise<Pagination<Submission<Product>> | null> {
 *     return null;
 *   }
 * }
 * ```
 *
 * And here is the Submission model:
 *
 * ```typescript
 * import {Generics, Property} from "@tsed/schema";
 *
 * @Generics("T")
 * class Submission<T> {
 *   @Property()
 *   _id: string;
 *   @Property("T")
 *   data: T;
 * }
 * ```
 *
 * @param status
 * @param model
 * @decorator
 * @swagger
 * @schema
 * @response
 * @operation
 */
export declare function Returns(status?: string | number, model?: Type<any> | Type<any>[]): ReturnsChainedDecorators;
